@@ -36,6 +36,7 @@
 #include <dbus/dbus-glib-lowlevel.h>
 
 #include <obex-agent.h>
+#include <bluetooth-client.h>
 #include <bluetooth-chooser.h>
 #include "marshal.h"
 
@@ -44,7 +45,9 @@
 static DBusGConnection *conn = NULL;
 
 static GtkWidget *dialog;
+static GtkWidget *button;
 static GtkWidget *label_from;
+static GtkWidget *image_status;
 static GtkWidget *label_status;
 static GtkWidget *progress;
 
@@ -62,8 +65,6 @@ static int file_index = 0;
 static gint64 first_update = 0;
 static gint64 last_update = 0;
 
-static DBusGProxy *session_proxy = NULL;
-
 static gchar *filename_to_path(const gchar *filename)
 {
 	GFile *file;
@@ -141,14 +142,15 @@ static gboolean is_palm_device(const gchar *bdaddr)
 
 static void create_window(void)
 {
-	GtkWidget *vbox;
+	GtkWidget *vbox, *hbox;
 	GtkWidget *table;
 	GtkWidget *label;
 	gchar *text;
 
 	dialog = gtk_dialog_new_with_buttons(_("File Transfer"), NULL,
 				GTK_DIALOG_NO_SEPARATOR,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+				NULL);
+	button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_window_set_type_hint(GTK_WINDOW(dialog),
 						GDK_WINDOW_TYPE_HINT_NORMAL);
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
@@ -210,10 +212,18 @@ static void create_window(void)
 							_("Connecting..."));
 	gtk_box_pack_start(GTK_BOX(vbox), progress, TRUE, TRUE, 0);
 
+	hbox = gtk_hbox_new (FALSE, 4);
+
+	image_status = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_MENU);
+	gtk_widget_set_no_show_all (image_status, TRUE);
+	gtk_box_pack_start(GTK_BOX (hbox), image_status, FALSE, FALSE, 4);
+
 	label_status = gtk_label_new(NULL);
 	gtk_misc_set_alignment(GTK_MISC(label_status), 0, 0.5);
 	gtk_label_set_ellipsize(GTK_LABEL(label_status), PANGO_ELLIPSIZE_END);
-	gtk_box_pack_start(GTK_BOX(vbox), label_status, TRUE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX (hbox), label_status, TRUE, TRUE, 4);
+
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 2);
 
 	g_signal_connect(G_OBJECT(dialog), "response",
 				G_CALLBACK(response_callback), NULL);
@@ -221,185 +231,6 @@ static void create_window(void)
 	gtk_widget_show_all(dialog);
 }
 
-static void finish_sending(DBusGProxy *proxy)
-{
-	gtk_label_set_markup(GTK_LABEL(label_status), NULL);
-
-	dbus_g_proxy_call(proxy, "Disconnect", NULL, G_TYPE_INVALID,
-							G_TYPE_INVALID);
-
-	gtk_widget_destroy(dialog);
-
-	gtk_main_quit();
-}
-
-static void send_file(DBusGProxy *proxy)
-{
-	gchar *filename = option_files[file_index];
-	GError *error = NULL;
-	gchar *basename, *text, *markup;
-
-	if (file_index > file_count - 1) {
-		finish_sending(proxy);
-		return;
-	}
-
-	if (filename[0] == '\0') {
-		file_index++;
-		send_file(proxy);
-		return;
-	}
-
-	text = g_path_get_dirname(filename);
-	gtk_label_set_markup(GTK_LABEL(label_from), text);
-	g_free(text);
-
-	basename = g_path_get_basename(filename);
-	text = g_strdup_printf(_("Sending %s"), basename);
-	g_free(basename);
-	markup = g_markup_printf_escaped("<i>%s</i>", text);
-	gtk_label_set_markup(GTK_LABEL(label_status), markup);
-	g_free(markup);
-	g_free(text);
-
-	text = g_strdup_printf(_("Sending file %d of %d"),
-						file_index + 1, file_count);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), text);
-	g_free(text);
-
-	dbus_g_proxy_call(proxy, "SendFile", &error,
-				G_TYPE_STRING, filename, G_TYPE_INVALID,
-							G_TYPE_INVALID);
-
-	if (error != NULL) {
-		g_printerr("Sending of file %s failed: %s\n", filename,
-							error->message);
-		g_error_free(error);
-		gtk_main_quit();
-	}
-}
-
-static void transfer_started(DBusGProxy *proxy, gchar *a, gchar *b,
-					guint64 size, gpointer user_data)
-{
-	current_size = size;
-
-	last_update = get_system_time();
-}
-
-static void transfer_progress(DBusGProxy *proxy,
-					guint64 bytes, gpointer user_data)
-{
-	gint64 current_time;
-	gint elapsed_time;
-	gint remaining_time;
-	gint transfer_rate;
-	guint64 current_sent;
-	gdouble fraction;
-	gchar *time, *rate, *file, *text;
-
-	current_sent = total_sent + bytes;
-	fraction = (gdouble) current_sent / (gdouble) total_size;
-	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), fraction);
-
-	current_time = get_system_time();
-	elapsed_time = (current_time - first_update) / 1000000;
-
-	if (current_time < last_update + 1000000)
-		return;
-
-	last_update = current_time;
-
-	if (elapsed_time == 0)
-		return;
-
-	transfer_rate = current_sent / elapsed_time;
-
-	if (transfer_rate == 0)
-		return;
-
-	remaining_time = (total_size - current_sent) / transfer_rate;
-
-	time = format_time(remaining_time);
-
-	if (transfer_rate >= 3000)
-		rate = g_strdup_printf(_("%d KB/s"), transfer_rate / 1000);
-	else
-		rate = g_strdup_printf(_("%d B/s"), transfer_rate);
-
-	file = g_strdup_printf(_("Sending file %d of %d"),
-						file_index + 1, file_count);
-	text = g_strdup_printf("%s (%s, %s)", file, rate, time);
-	g_free(file);
-	g_free(rate);
-	g_free(time);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), text);
-	g_free(text);
-}
-
-static void transfer_completed(DBusGProxy *proxy, gpointer user_data)
-{
-	total_sent += current_size;
-
-	file_index++;
-
-	send_file(proxy);
-}
-
-static void transfer_cancelled(DBusGProxy *proxy, gpointer user_data)
-{
-	transfer_completed(proxy, user_data);
-}
-
-static void error_occurred(DBusGProxy *proxy, const gchar *name,
-				const gchar *message, gpointer user_data)
-{
-	gchar *text;
-
-	g_return_if_fail (proxy == session_proxy);
-
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress),
-						_("Error Occurred"));
-
-	text = g_strdup_printf("<span foreground=\"red\">%s</span>", message);
-	gtk_label_set_markup(GTK_LABEL(label_status), text);
-	g_free(text);
-
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-						GTK_RESPONSE_CLOSE, TRUE);
-}
-
-static void session_connect_error (DBusGProxy *proxy, const char *session_obj, const char *error_name,
-				   const char *error_message, gpointer user_data)
-{
-	gchar *text;
-
-	if (strcmp (session_obj, dbus_g_proxy_get_path (session_proxy)) != 0)
-		return;
-
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress),
-						_("Error Occurred"));
-
-	text = g_strdup_printf("<span foreground=\"red\">%s</span>", error_message);
-	gtk_label_set_markup(GTK_LABEL(label_status), text);
-	g_free(text);
-
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-						GTK_RESPONSE_CLOSE, TRUE);
-}
-
-static void session_connected(DBusGProxy *proxy, const char *session_obj, gpointer user_data)
-{
-	if (strcmp (session_obj, dbus_g_proxy_get_path (session_proxy)) != 0)
-		return;
-
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), NULL);
-
-	first_update = get_system_time();
-
-	send_file(session_proxy);
-}
-
 #define OPENOBEX_CONNECTION_FAILED "org.openobex.Error.ConnectionAttemptFailed"
 
 static gchar *get_error_message(GError *error)
@@ -432,151 +263,59 @@ done:
 	return message;
 }
 
-static void create_notify(DBusGProxy *proxy,
-				DBusGProxyCall *call, void *user_data)
-{
-	GError *error = NULL;
-	const gchar *path = NULL;
-
-	if (dbus_g_proxy_end_call(proxy, call, &error,
-					DBUS_TYPE_G_OBJECT_PATH, &path,
-						G_TYPE_INVALID) == FALSE) {
-		gchar *message;
-
-		message = get_error_message(error);
-		error_occurred(proxy, NULL, message, NULL);
-		g_free(message);
-
-		return;
-	}
-
-	session_proxy = dbus_g_proxy_new_for_name(conn, "org.openobex",
-						path, "org.openobex.Session");
-
-	dbus_g_proxy_add_signal(session_proxy, "ErrorOccurred",
-				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal(session_proxy, "ErrorOccurred",
-				G_CALLBACK(error_occurred), NULL, NULL);
-
-	dbus_g_proxy_add_signal(session_proxy, "Cancelled", G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal(session_proxy, "Cancelled",
-				G_CALLBACK(transfer_cancelled), NULL, NULL);
-
-	dbus_g_proxy_add_signal(session_proxy, "TransferStarted", G_TYPE_STRING,
-				G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal(session_proxy, "TransferStarted",
-				G_CALLBACK(transfer_started), NULL, NULL);
-
-	dbus_g_proxy_add_signal(session_proxy, "TransferProgress",
-						G_TYPE_UINT64, G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal(session_proxy, "TransferProgress",
-				G_CALLBACK(transfer_progress), NULL, NULL);
-
-	dbus_g_proxy_add_signal(session_proxy, "TransferCompleted", G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal(session_proxy, "TransferCompleted",
-				G_CALLBACK(transfer_completed), NULL, NULL);
-}
-
-static gchar *get_name(DBusGProxy *device)
-{
-	GHashTable *hash;
-
-	if (dbus_g_proxy_call(device, "GetProperties", NULL,
-			G_TYPE_INVALID, dbus_g_type_get_map("GHashTable",
-						G_TYPE_STRING, G_TYPE_VALUE),
-					&hash, G_TYPE_INVALID) != FALSE) {
-		GValue *value;
-		gchar *name;
-
-		value = g_hash_table_lookup(hash, "Name");
-		name = value ? g_value_dup_string(value) : NULL;
-		g_hash_table_destroy(hash);
-
-		return name;
-	}
-
-	return NULL;
-}
-
 static gchar *get_device_name(const gchar *address)
 {
-	DBusGConnection *connection;
-	DBusGProxy *manager;
-	GPtrArray *adapters;
-	gchar *name;
-	guint i;
+	BluetoothClient *client;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean cont;
+	char *found_name;
 
-	name = NULL;
-
-	connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, NULL);
-	if (connection == NULL)
-		return name;
-
-	manager = dbus_g_proxy_new_for_name(connection, "org.bluez",
-						"/", "org.bluez.Manager");
-	if (manager == NULL) {
-		dbus_g_connection_unref(connection);
-		return name;
+	found_name = NULL;
+	client = bluetooth_client_new ();
+	model = bluetooth_client_get_model (client);
+	if (model == NULL) {
+		g_object_unref (client);
+		return NULL;
 	}
 
-	if (dbus_g_proxy_call(manager, "ListAdapters", NULL,
-			G_TYPE_INVALID, dbus_g_type_get_collection("GPtrArray",
-						DBUS_TYPE_G_OBJECT_PATH),
-					&adapters, G_TYPE_INVALID) == FALSE) {
-		g_object_unref(manager);
-		dbus_g_connection_unref(connection);
-		return name;
-	}
+	cont = gtk_tree_model_get_iter_first(model, &iter);
+	while (cont != FALSE) {
+		char *bdaddr, *name;
 
-	for (i = 0; i < adapters->len && name == NULL; i++) {
-		DBusGProxy *adapter;
-		char *device_path;
-
-		adapter = dbus_g_proxy_new_for_name(connection, "org.bluez",
-						g_ptr_array_index(adapters, i),
-							"org.bluez.Adapter");
-
-		if (dbus_g_proxy_call(adapter, "FindDevice", NULL,
-					G_TYPE_STRING, address, G_TYPE_INVALID,
-					DBUS_TYPE_G_OBJECT_PATH, &device_path,
-						G_TYPE_INVALID) == TRUE) {
-			DBusGProxy *device;
-			device = dbus_g_proxy_new_for_name(connection,
-						"org.bluez", device_path,
-							"org.bluez.Device");
-			name = get_name(device);
-			g_object_unref(device);
+		gtk_tree_model_get(model, &iter,
+				   BLUETOOTH_COLUMN_ADDRESS, &bdaddr,
+				   BLUETOOTH_COLUMN_ALIAS, &name,
+				   -1);
+		if (g_strcmp0 (bdaddr, address) == 0) {
+			g_free (bdaddr);
+			found_name = name;
 			break;
 		}
+		g_free (bdaddr);
+		g_free (name);
 
-		g_object_unref(adapter);
+		cont = gtk_tree_model_iter_next(model, &iter);
 	}
 
-	g_ptr_array_free(adapters, TRUE);
-	g_object_unref(manager);
+	g_object_unref (model);
+	g_object_unref (client);
 
-	dbus_g_connection_unref(connection);
-
-	return name;
+	return found_name;
 }
 
-static gboolean request_callback(DBusGMethodInvocation *context,
-				DBusGProxy *transfer, gpointer user_data)
+static void get_properties_callback (DBusGProxy *proxy,
+				     DBusGProxyCall *call,
+				     void *user_data)
 {
+	GError *error = NULL;
 	gchar *filename = option_files[file_index];
 	gchar *basename, *text, *markup;
 	GHashTable *hash;
 
-	if (dbus_g_proxy_call(transfer,
-				"GetProperties", NULL, G_TYPE_INVALID,
-				dbus_g_type_get_map("GHashTable",
-						G_TYPE_STRING, G_TYPE_VALUE),
-					&hash, G_TYPE_INVALID) == TRUE) {
+	if (dbus_g_proxy_end_call (proxy, call, &error,
+				   dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), &hash,
+				   G_TYPE_INVALID) != FALSE) {
 		GValue *value;
 
 		value = g_hash_table_lookup(hash, "Size");
@@ -605,6 +344,14 @@ static gboolean request_callback(DBusGMethodInvocation *context,
 						file_index + 1, file_count);
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), text);
 	g_free(text);
+}
+
+static gboolean request_callback(DBusGMethodInvocation *context,
+				DBusGProxy *transfer, gpointer user_data)
+{
+	dbus_g_proxy_begin_call(transfer, "GetProperties",
+				get_properties_callback, NULL, NULL,
+				G_TYPE_INVALID);
 
 	dbus_g_method_return(context, "");
 
@@ -615,8 +362,56 @@ static gboolean progress_callback(DBusGMethodInvocation *context,
 				DBusGProxy *transfer, guint64 transferred,
 							gpointer user_data)
 {
-	transfer_progress(NULL, transferred, NULL);
+	gint64 current_time;
+	gint elapsed_time;
+	gint remaining_time;
+	gint transfer_rate;
+	guint64 current_sent;
+	gdouble fraction;
+	gchar *time, *rate, *file, *text;
 
+	current_sent = total_sent + transferred;
+	if (total_size == 0)
+		fraction = 0.0;
+	else
+		fraction = (gdouble) current_sent / (gdouble) total_size;
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), fraction);
+
+	current_time = get_system_time();
+	elapsed_time = (current_time - first_update) / 1000000;
+
+	if (current_time < last_update + 1000000)
+		goto done;
+
+	last_update = current_time;
+
+	if (elapsed_time == 0)
+		goto done;
+
+	transfer_rate = current_sent / elapsed_time;
+
+	if (transfer_rate == 0)
+		goto done;
+
+	remaining_time = (total_size - current_sent) / transfer_rate;
+
+	time = format_time(remaining_time);
+
+	if (transfer_rate >= 3000)
+		rate = g_strdup_printf(_("%d KB/s"), transfer_rate / 1000);
+	else
+		rate = g_strdup_printf(_("%d B/s"), transfer_rate);
+
+	file = g_strdup_printf(_("Sending file %d of %d"),
+						file_index + 1, file_count);
+	text = g_strdup_printf("%s (%s, %s)", file, rate, time);
+	g_free(file);
+	g_free(rate);
+	g_free(time);
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), text);
+	g_free(text);
+
+done:
 	dbus_g_method_return(context);
 
 	return TRUE;
@@ -629,6 +424,8 @@ static gboolean complete_callback(DBusGMethodInvocation *context,
 
 	file_index++;
 
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 1.0);
+
 	dbus_g_method_return(context);
 
 	return TRUE;
@@ -655,19 +452,16 @@ static void send_notify(DBusGProxy *proxy,
 
 	if (dbus_g_proxy_end_call(proxy, call, &error,
 						G_TYPE_INVALID) == FALSE) {
-		gchar *text, *message;
+		char *message;
 
 		message = get_error_message(error);
+		gtk_widget_show (image_status);
+		gtk_label_set_markup(GTK_LABEL(label_status), message);
+		g_free (message);
 
-		text = g_strdup_printf("<span foreground=\"red\">%s</span>",
-								message);
-		gtk_label_set_markup(GTK_LABEL(label_status), text);
-		g_free(text);
-
-		g_free(message);
-
+		gtk_button_set_label (GTK_BUTTON (button), GTK_STOCK_CLOSE);
 		gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-						GTK_RESPONSE_CLOSE, TRUE);
+						GTK_RESPONSE_CANCEL, TRUE);
 		return;
 	}
 
@@ -682,13 +476,6 @@ static void value_free(GValue *value)
 	g_free(value);
 }
 
-static void name_owner_changed(DBusGProxy *proxy, const char *name,
-			const char *prev, const char *new, gpointer user_data)
-{
-	if (g_str_equal(name, "org.openobex") == TRUE && *new == '\0')
-		gtk_main_quit();
-}
-
 static void
 select_device_changed(BluetoothChooser *sel,
 		      char *address,
@@ -703,7 +490,7 @@ select_device_changed(BluetoothChooser *sel,
 static char *
 show_browse_dialog (char **device_name)
 {
-	GtkWidget *dialog, *selector, *button, *image;
+	GtkWidget *dialog, *selector, *send_button, *image;
 	char *bdaddr;
 	int response_id;
 
@@ -711,9 +498,9 @@ show_browse_dialog (char **device_name)
 					     GTK_DIALOG_NO_SEPARATOR,
 					     GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 					     NULL);
-	button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("Send _To"), GTK_RESPONSE_ACCEPT);
+	send_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("Send _To"), GTK_RESPONSE_ACCEPT);
 	image = gtk_image_new_from_icon_name ("document-send", GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image (GTK_BUTTON (button), image);
+	gtk_button_set_image (GTK_BUTTON (send_button), image);
 	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
 					  GTK_RESPONSE_ACCEPT, FALSE);
 	gtk_window_set_default_size(GTK_WINDOW(dialog), 480, 400);
@@ -736,11 +523,10 @@ show_browse_dialog (char **device_name)
 
 	bdaddr = NULL;
 	response_id = gtk_dialog_run (GTK_DIALOG (dialog));
-	if (response_id == GTK_RESPONSE_ACCEPT)
-		g_object_get (G_OBJECT (selector),
-			      "device-selected", &bdaddr,
-			      "device-selected-name", device_name,
-			      NULL);
+	if (response_id == GTK_RESPONSE_ACCEPT) {
+		bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (selector));
+		*device_name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (selector));
+	}
 
 	gtk_widget_destroy (dialog);
 
@@ -784,7 +570,7 @@ static GOptionEntry options[] = {
 	{ "device", 0, 0, G_OPTION_ARG_STRING, &option_device,
 				N_("Remote device to use"), "ADDRESS" },
 	{ "name", 0, 0, G_OPTION_ARG_STRING, &option_device_name,
-				N_("Remove device's name"), NULL },
+				N_("Remote device's name"), NULL },
 	{ "dest", 0, G_OPTION_FLAG_HIDDEN,
 			G_OPTION_ARG_STRING, &option_device, NULL, NULL },
 	{ G_OPTION_REMAINING, 0, 0,
@@ -795,6 +581,9 @@ static GOptionEntry options[] = {
 int main(int argc, char *argv[])
 {
 	DBusGProxy *proxy;
+	GHashTable *hash = NULL;
+	GValue *value;
+	ObexAgent *agent;
 	GError *error = NULL;
 	int i;
 
@@ -901,66 +690,32 @@ int main(int argc, char *argv[])
 
 	create_window();
 
-	proxy = dbus_g_proxy_new_for_name_owner(conn, "org.openobex.client",
-					"/", "org.openobex.Client", NULL);
+	proxy = dbus_g_proxy_new_for_name(conn, "org.openobex.client",
+					"/", "org.openobex.Client");
 
-	if (proxy == NULL) {
-		proxy = dbus_g_proxy_new_for_name(conn, "org.openobex",
-				"/org/openobex", "org.openobex.Manager");
+	agent = obex_agent_new();
 
-		dbus_g_proxy_add_signal(proxy, "NameOwnerChanged",
-					G_TYPE_STRING, G_TYPE_STRING,
-					G_TYPE_STRING, G_TYPE_INVALID);
+	obex_agent_set_release_func(agent, release_callback, NULL);
+	obex_agent_set_request_func(agent, request_callback, NULL);
+	obex_agent_set_progress_func(agent, progress_callback, NULL);
+	obex_agent_set_complete_func(agent, complete_callback, NULL);
 
-		dbus_g_proxy_connect_signal(proxy, "NameOwnerChanged",
-				G_CALLBACK(name_owner_changed), NULL, NULL);
+	obex_agent_setup(agent, AGENT_PATH);
 
-		dbus_g_proxy_add_signal(proxy, "SessionConnected", DBUS_TYPE_G_OBJECT_PATH, G_TYPE_INVALID);
+	hash = g_hash_table_new_full(g_str_hash, g_str_equal,
+				     g_free, (GDestroyNotify) value_free);
 
-		dbus_g_proxy_connect_signal(proxy, "SessionConnected",
-					    G_CALLBACK(session_connected), NULL, NULL);
+	value = g_new0(GValue, 1);
+	g_value_init(value, G_TYPE_STRING);
+	g_value_set_string(value, option_device);
+	g_hash_table_insert(hash, g_strdup("Destination"), value);
 
-		dbus_g_proxy_add_signal(proxy, "SessionConnectError",
-					DBUS_TYPE_G_OBJECT_PATH, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
-
-		dbus_g_proxy_connect_signal(proxy, "SessionConnectError",
-					    G_CALLBACK(session_connect_error), NULL, NULL);
-
-		dbus_g_proxy_begin_call(proxy, "CreateBluetoothSession",
-					create_notify, NULL, NULL,
-					G_TYPE_STRING, option_device,
-					G_TYPE_STRING, "00:00:00:00:00:00",
-					G_TYPE_STRING, "opp", G_TYPE_INVALID);
-	} else {
-		GHashTable *hash = NULL;
-		GValue *value;
-		ObexAgent *agent;
-
-		agent = obex_agent_new();
-
-		obex_agent_set_release_func(agent, release_callback, NULL);
-		obex_agent_set_request_func(agent, request_callback, NULL);
-		obex_agent_set_progress_func(agent, progress_callback, NULL);
-		obex_agent_set_complete_func(agent, complete_callback, NULL);
-
-		obex_agent_setup(agent, AGENT_PATH);
-
-		hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-					g_free, (GDestroyNotify) value_free);
-
-		value = g_new0(GValue, 1);
-		g_value_init(value, G_TYPE_STRING);
-		g_value_set_string(value, option_device);
-		g_hash_table_insert(hash, g_strdup("Destination"), value);
-
-		dbus_g_proxy_begin_call(proxy, "SendFiles",
+	dbus_g_proxy_begin_call(proxy, "SendFiles",
 				send_notify, NULL, NULL,
-				dbus_g_type_get_map("GHashTable",
-					G_TYPE_STRING, G_TYPE_VALUE), hash,
+				dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), hash,
 				G_TYPE_STRV, option_files,
 				DBUS_TYPE_G_OBJECT_PATH, AGENT_PATH,
-							G_TYPE_INVALID);
-	}
+				G_TYPE_INVALID);
 
 	gtk_main();
 
